// sources.h
// Copyright (C) 2004-2011 COOLSTF.com.
//
// Include file for developing TSReader sources
//
#include <windows.h>

// Size of transport stream buffers - do NOT change
// 256 elements in the ring buffer
// up to 698 x 188 per buffer. We chose this as it's close to 128KB.
// So 256 x 698 x 188 = 33593344 or about 32MB for the TS buffers
#define MAX_TS_BUFFERS 256
#define TS_PACKETS_AT_A_TIME 698
#define TS_BUFFER_SIZE 188 * TS_PACKETS_AT_A_TIME

typedef struct tag_TSBuffers
{
	int nSize;					// size of data in the buffer. Always < TS_BUFFER_SIZE. Filled by the source
	BYTE * pData;				// pointer to buffer of size TS_BUFFER_SIZE. TS data is put there by the source
	DWORD * pTimestamps;		// pointer to 4 x TS_PACKETS_AT_A_TIME for timestamps. Filled by the source.
} TSBUFFERS, *PTSBUFFERS;

typedef struct _tagSourceStruct
{
	// The following group need to be setup after a tune - TSReader saves these
	// in the registry for you
	BOOL fSpectrumInversion;	// spectral inversion status
	int nFrequency;				// frequency to tune (ATSC/QAM/Satellite in MHz, DVB-T and DVB-C in KHz)
	int nPolarity;				// polarity for satellite: 0 = Vertical/RHCP 1 = Horizontal/LHCP -1 = no power
	int nSymbolRate;			// symbol rate
	int nLNBFrequency;			// LNB frequency
	int n22KHz;					// 22 KHz tone status
	int nDiSEqCInput;			// Switch input 0 = none, 1-4 = DiSEqC, 5 = TBA, 6 = TBB 7-20 = Dish switch (see below)
	int nBandwidth;				// Signal bandwidth 0 = 6MHz, 1 = 7MHz, 2 = 8 MHz
	int nADVModulationMode;		// Satellite modulation mode (see below)
	int nCodeRate;				// Code rate index
	int nQAM;					// QAM modulation mode 0 = QAM-16, 1 = QAM-32, 2 = QAM-64, 3 = QAM-128, 4 = QAM-256
	int nUDPMulticastPort;		// UDP source only: multicast port
	int nSourceIndex;			// Source device offset for multiple board support
	char szUDPMulticastAddress[MAX_PATH];	// UDP source only: multicast address
	char szUDPMulticastInterface[MAX_PATH];	// UDP source only: interface address

	// These are used for data flow from the source into TSReader
	int nTSBuffersInUse;		// Current buffers in use - bump by 1 each time a buffer is added
	int nLastSecondByteCounter;	// bump as data is received
	int nInputThreadPriority;	// priority input thread should run at 0 = THREAD_PRIORITY_NORMAL, 1 = THREAD_PRIORITY_HIGHEST, 2 = THREAD_PRIORITY_IDLE
	CRITICAL_SECTION csTSBuffersInUse;	// critical section to protect nTSBuffersInUse
	CRITICAL_SECTION csPIDCounter; // critical section to protect nLastSecondByteCounter
	TSBUFFERS tsb[MAX_TS_BUFFERS]; // transport stream buffer structs

	// Misc. flags between TSReader and the source
	BOOL fDontTune;				// flag set by the registry to tell the source not to tune - debug use mostly
	BOOL fReadThreadTerminated;	// set TRUE when the read thread is done
	BOOL fTerminateReadThread;	// flag from TSReader to terminate the read thread
	BOOL fSerialReceiverControlEnabled;	// serial control is enabled - you should use the sourcehelper functions
	BOOL fQuietMode;			// quiet mode - don't output "can't tune" messages
	BOOL fIgnoreContinuity;		// tells TSReader to ignore continuity errors in the stream
	BOOL fTimestampPackets;		// set if TSReader wants timestamps sent back with packets. Source needs CAPABILITIES_TIMESTAMP
	int nPATCATProcessed;		// sets to 3 once the PAT and CAT have been parsed by TSReader
								// this is useful to feed small buffers to TSReader when it starts
								// running with a demux interface.
	HANDLE hReadDataThread;		// handle of the thread sending data to TSReader (set by source)
	HWND hWndTSReader;			// TSReader's main window - use as the parent window for you windows/messages
	HINSTANCE hTSReaderInst;	// TSReader's resource handle

	// These are used by the file source only
	BOOL fLastFileTS;			// flags if the last file as .ts or .mpg
	char szDropFilename[MAX_PATH];	// filenames dropped onto the TSReader main window
	char szTransportStreamInitialDir[MAX_PATH];	// initial directory for transport stream files

} SOURCESTRUCT, *PSOURCESTRUCT;

// Satellite modulation modes
#define ADV_MOD_DVB_QPSK 0						// DVB-S QPSK
#define ADV_MOD_TURBO_QPSK 1					// Turbo QPSK
#define ADV_MOD_TURBO_8PSK 2					// Turbo 8PSK (also used for Trellis 8PSK DVB-SNG)
#define ADV_MOD_TURBO_16QAM 3					// Turbo 16QAM (also used for Trellis 16QAM DVB-SNG)
#define ADV_MOD_DCII_C_QPSK 4					// Digicipher II Combo
#define ADV_MOD_DCII_I_QPSK 5					// Digicipher II I-stream
#define ADV_MOD_DCII_Q_QPSK 6					// Digicipher II Q-stream
#define ADV_MOD_DCII_C_OQPSK 7					// Digicipher II offset QPSK
#define ADV_MOD_DVBS2 8							// DVB-S2
#define ADV_MOD_DSS_QPSK 9						// DSS (DIRECTV) QPSK
#define ADV_MOD_DVB_BPSK 10						// DVB-S BPSK

// Satellite modulation options (bit values)
#define ADV_MODB_DVB2_PILOT 0x00010000			// DVB-S2 pilot detection enabled
#define ADV_MODB_DVB2_RO020 0x00020000			// DVB-S2 rolloff a=0.20
#define ADV_MODB_DVB2_RO025 0x00040000			// DVB-S2 rolloff a=0.25

// Source capabilities
#define CAPABILITIES_DISEQC				0x1		// source can send DiSEqC commands
#define CAPABILITIES_TONEBURST			0x2		// source can send toneburst switch commands
#define CAPABILITIES_POWER				0x4		// source provides power (satellite type)
#define CAPABILITIES_SERIAL_CONTROL		0x8		// source can use TSReader's serial control
#define CAPABILITIES_UNIPROCESSOR		0x10	// source not MP safe so is run with only 1 processor
#define CAPABILITIES_DISEQC_POSITIONER	0x20	// source can send DiSEqC positioner commands
#define CAPABILITIES_DISH_SWITCH		0x40	// source supports Dish Network switch commands
#define CAPABILITIES_ADV_SATELLITE		0x80	// source handles advanced modulation (DVB-S + DCII and Turbo modes)
#define CAPABILITIES_TIMESTAMP			0x100	// source can timestamp packets
#define CAPABILITIES_MULTICARD			0x200	// source supports multiple cards
#define CAPABILITIES_CI_CAM				0x400	// source supports Common Interface CAMs
#define CAPABILITIES_TUNE_BY_CHANNEL	0x800	// source tunes by the channel number
#define CAPABILITIES_ACTIVE_ANTENNA		0x1000	// source has an active antenna controller
#define CAPABILITIES_MULTICARD_DISPLAY	0x2000	// source supports display of multiple cards
#define CAPABILITIES_SIGNALLED			0x4000	// if set, fLastFileTS becomes an event and buffer availability is signalled through the Win32 API
#define CAPABILITIES_AUTOSCAN			0x8000	// source supports scanning

// Sources with CAPABILITIES_DISH_SWITCH support Dish Network legacy switches with the following values:
//  7 - SW21 Dish 1
//  8 - SW21 Dish 2
//  9 - SW42 Dish 1
// 10 - SW42 Dish 2
// 11 - SW44 Dish 2
// 12 - SW64 Dish 1A
// 13 - SW64 Dish 1B
// 14 - SW64 Dish 2A
// 15 - SW64 Dish 2B
// 16 - SW64 Dish 3A
// 17 - SW64 Dish 3B
// 18 - Twin LNB 1
// 19 - Twin LNB 2
// 20 - Quad LNB 2

// Routines exported from TSReader_SourceHelper.dll
#ifdef  __cplusplus
extern "C" {
#endif

// Sets priorities of the threads used by TSReader
void __cdecl SourceHelper_SetWorkerThreadPriorities(BOOL fStreamProcessingThread);

// Conversion utilities (QAM/ATSC for USA bandplan only)
void __cdecl SourceHelper_ConvertPolarity(char * szPolarity);
int __cdecl  SourceHelper_GetFrequencyFromATSCChannel(int nChannel);
int __cdecl  SourceHelper_GetFrequencyFromQAMChannel(int nChannel);
int __cdecl  SourceHelper_GetQAMChannelFromFrequency(int nFrequency);
int __cdecl  SourceHelper_GetATSCChannelFromFrequency(int nFrequency);

// Tuner dialogs - these are called by the source module to present a standard
// tuning interface to the user. Although its possible to use your own dialog
// to get tuner parameters, we suggest using TSReader's dialogs.
BOOL __cdecl SourceHelper_DVBSTuneDialog(HWND hWnd);			//  DVB-S signals
BOOL __cdecl SourceHelper_DVBTTuneDialog(HWND hWnd);			//  DVB-T signals
BOOL __cdecl SourceHelper_DVBCTuneDialog(HWND hWnd);			//  DVB-T signals
BOOL __cdecl SourceHelper_DSSTuneDialog(HWND hWnd);				//  DIRECTV Ku-band signals
BOOL __cdecl SourceHelper_ATSCTuneDialog(HWND hWnd);			//  ATSC signals
BOOL __cdecl SourceHelper_ISDBTTuneDialog(HWND hWnd);			//  ISDB-T signals
BOOL __cdecl SourceHelper_ADVTuneDialog(HWND hWnd);				//  advanced satellite tuners
BOOL __cdecl SourceHelper_QAMTuneDialog(HWND hWnd);				//  SCTE QAM signals
BOOL __cdecl SourceHelper_UDPMulticastTuneDialog(HWND hWnd);	//  Multicast UDP streams
BOOL __cdecl SourceHelper_UDPUnicastTuneDialog(HWND hWnd);		//  Unicast UDP streams
BOOL __cdecl SourceHelper_TCPTuneDialog(HWND hWnd);				//  TCP delivered streams

// Functions for serial receiver control modules
BOOL __cdecl SourceHelper_TuneSerialControl(char * szTunerString);
void __cdecl SourceHelper_SerialControlStart();
void __cdecl SourceHelper_SerialControlStop();
BOOL __cdecl SourceHelper_SerialControlGetSignal(char * szSignalString);
BYTE __cdecl SourceHelper_ReceiveReceiverSerial(int * nTimeout);
int  __cdecl SourceHelper_SendReceiverSerial(unsigned char * lpByte, DWORD dwBytesToWrite);

// Sync functions
BOOL __cdecl SourceHelper_StartSyncThread(PSOURCESTRUCT pss, BOOL fDSSMode);
BOOL __cdecl SourceHelper_StopSyncThread();
BOOL __cdecl SourceHelper_SyncData(BYTE * pData, int nLength);
int  __cdecl SourceHelper_GetSyncLossCount(BOOL fReset);

// Misc functions
BOOL __cdecl SourceHelper_GetQAMHRCStatus();
BOOL __cdecl SourceHelper_myGetOpenFileName(OPENFILENAME * lpofn);
BOOL __cdecl SourceHelper_myGetOpenFileNameW(LPOPENFILENAMEW lpofn);
BOOL __cdecl SourceHelper_GetTSReaderVersion(int * nMajor, int * nMinor, int * nBuild);
BOOL __cdecl SourceHelper_GetProfileName(char * szBuffer);
void __cdecl SourceHelper_OutputDebugString(char * szDebugString);
HANDLE __cdecl SourceHelper_GetSourceBufferEventHandle();

// Command-line parsing functions
BOOL __cdecl SourceHelper_Parse_CommandLine_ATSC(char * szCommandLine, BOOL fQuiet, int * nFrequency);
BOOL __cdecl SourceHelper_Parse_CommandLine_QAM(char * szCommandLine, BOOL fQuiet, int * nFrequency);
BOOL __cdecl SourceHelper_Parse_CommandLine_DVBS(char * szCommandLine, BOOL fQuiet, int * nFrequency, int * nPolarity, int * nSymbolRate, int * nLNBFrequency, int * n22KHz, int * nDiSEqCInput);
BOOL __cdecl SourceHelper_Parse_CommandLine_DVBT(char * szCommandLine, BOOL fQuiet, int * nFrequency, BOOL * fSpectrumInversion, int * nBandwidth);
BOOL __cdecl SourceHelper_Parse_CommandLine_DVBC(char * szCommandLine, BOOL fQuiet, int * nFrequency, int * nSymbolRate, int * nQAM, BOOL * fSpectrumInversion, int * nBandwidth);
BOOL __cdecl SourceHelper_Parse_CommandLine_ADV(char * szCommandLine, BOOL fQuiet, int * nFrequency, int * nPolarity, int * nSymbolRate, int * nLNBFrequency, int * n22KHz, int * nADVModulationMode, int * nCodeRate, int * nDiSEqCInput);

#ifdef  __cplusplus
}
#endif
